// This file is part of Eigen, a lightweight C++ template library
// for linear algebra. Eigen itself is part of the KDE project.
//
// Copyright (C) 2006 Benoit Jacob <jacob@math.jussieu.fr>
//
// Eigen is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 or (at your option) any later version.
//
// Eigen is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along
// with Eigen; if not, write to the Free Software Foundation, Inc., 51
// Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. This exception does not invalidate any other reasons why a work
// based on this file might be covered by the GNU General Public License.

#ifndef EIGEN_TESTS_MAIN_H
#define EIGEN_TESTS_MAIN_H

#include <QtTest/QtTest>
#include <iostream>
#include <complex>
#include <stdlib.h>
#include <time.h>

#include "../matrix.h"

#define REPEAT 20

using namespace std;
using namespace Eigen;

class MainTest : public QObject
{
    Q_OBJECT

public:
    MainTest();

private slots:
    void checkBasicStuff();
    void checkRowColumnAdjoint();
    void checkLUDecomposition();
    void checkImplicitLU();
    void checkSolver();
    void checkRegression();
};

// these tests really aren't about precision. If the result is remotely sane,
// it's good enough.
template<typename T> inline bool test_isNegligible( const T& a, const T& b )
{
    return( a <= b * static_cast<T>(1000) * Util::epsilon(b) );
}

template<typename T> inline bool test_isApprox( const T& a, const T& b )
{
    return test_isNegligible( Util::abs( a - b ),
                              Util::min( Util::abs(a), Util::abs(b) ) );
}
inline void pickRandom( float & x )
{
    x = 2.0f * rand() / RAND_MAX - 1.0f;
}

inline void pickRandom( double & x )
{
    x = 2.0 * rand() / RAND_MAX - 1.0;
}

template<typename T> void pickRandom( std::complex<T> & x )
{
    pickRandom( x.real() );
    pickRandom( x.imag() );
}

template<typename MatrixType> void pickRandomMatrix( MatrixType & m )
{
    for( int i = 0; i < m.size(); i++ )
        for( int j = 0; j < m.size(); j++ )
            pickRandom( m(i,j) );
}

template<typename VectorType> void pickRandomVector( VectorType & v )
{
    for( int i = 0; i < v.size(); i++ )
            pickRandom( v(i) );
}

template<typename VectorType> bool compareVectors
( const VectorType & v1, const VectorType & v2, const char *strType )
{
    if( ! test_isNegligible( Util::abs( (v1 - v2).norm() ), Util::abs( v1.norm() ) ) ) goto _false;
    return true;
_false:
    qDebug() <<"Type =" << strType
             <<", relative error:" << Util::abs( (v1 - v2).norm() / v1.norm() );
    return false;
}

template<typename MatrixType> bool compareMatrices
( const MatrixType & m1, const MatrixType & m2, const char *strType )
{
    int i;
    typename MatrixType::Vector v1, v2;
    for( i = 0; i < m1.size(); i++ )
    {
        v1 = m1.columnVector(i);
        v2 = m2.columnVector(i);
        if( ! test_isNegligible( Util::abs( (v1 - v2).norm() ), Util::abs( v1.norm() ) ) ) goto _false;
    }
    return true;
_false:
    qDebug() <<"Type =" << strType
             <<", column =" << i
             <<", relative error:" << Util::abs( (v1 - v2).norm() / v1.norm() );
    return false;
}

#endif
